//
//  RelevantTestcases.m
//  linenumber_plugin
//
//  Created by Henning Kiel on 30.04.09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "RelevantTestcases.h"
#import "RegexKitLite.h"

@interface RelevantTestcases(private)
- (NSString *)relativeSourceFilePath;
- (NSString *)calculateMethodName;
- (void)calculateRelevantTestCases;
@end

@implementation RelevantTestcases

- (id)init;
{
  self = [super init];
  if (self)
  {
    lineNumber = [[NSNumber alloc] initWithInt:0];
    filePath = nil;
  }
  return self;
}

- (void)dealloc;
{
  [lineNumber release];
  [filePath release];
  [coverageData release];
  
  [super dealloc];
}


- (void)setLineNumber:(NSNumber *)aLineNumber;
{
  if (shouldIgnoreLineChangeUntilFileChange)
    return;
  
  NSLog(@"setLineNumber called with %@", aLineNumber);
  
  if (oldLineNumber != nil)
    [oldLineNumber autorelease];

  oldLineNumber = [lineNumber retain];
  [lineNumber autorelease];
  lineNumber = [aLineNumber retain];

  [self calculateRelevantTestCases];
}

- (void)setSourceFilePath:(NSString *)aFilePath;
{
  shouldIgnoreLineChangeUntilFileChange = false;
  
  [filePath autorelease];
  filePath = [aFilePath retain];
  
  [self calculateRelevantTestCases];
}

- (NSString *)relativeSourceFilePath;
{
  return [filePath stringByReplacingOccurrencesOfString:[[coverageData objectForKey:@"metadata"] objectForKey:@"base_dir"] withString:@""];
}

- (NSString *)calculateMethodName;
{
  NSString *source = [NSString stringWithContentsOfFile:filePath];
  if (source == nil)
    return nil;

  NSArray *lines = [source componentsSeparatedByString:@"\n"];

  NSInteger currentLineNr = [lineNumber unsignedIntValue];
  if (currentLineNr > [lines count] - 1)
    return nil;
      
  Boolean found = false;
  NSString *methodName = nil;
  for (currentLineNr; currentLineNr >= 0; currentLineNr--)
  {
    NSString *line = [lines objectAtIndex:currentLineNr];
    methodName = [line stringByMatching:@"(^def | def )(.*)$" capture:2];
    if (methodName != NULL)
    {
      found = true;
      break;
    }
  }
  
  return methodName;
}

- (void)calculateRelevantTestCases;
{
  NSArray *newRelevantTestCases = [NSArray array];
  
  if (coverageData == nil || [coverageData count] == 0)
  {
    [self setRelevantTestCases:newRelevantTestCases];
    return;
  }
      
  NSString *methodName = [self calculateMethodName];
  if (methodName == nil)
  {
    [self setRelevantTestCases:newRelevantTestCases];
    return;
  }
  
  NSDictionary *data = [coverageData objectForKey:@"data"];
  NSArray *dataForFileAndMethod = [[data objectForKey:[self relativeSourceFilePath]] objectForKey:methodName];
  newRelevantTestCases = [dataForFileAndMethod sortedArrayUsingDescriptors:[NSArray arrayWithObject:[[[NSSortDescriptor alloc] initWithKey:@"hits" ascending:NO] autorelease]]];
  [self setRelevantTestCases:newRelevantTestCases];
}

- (void)setRelevantTestCases:(NSArray *)newTestCases;
{
  [self willChangeValueForKey:@"relevantTestCases"];
  [relevantTestCases autorelease];
  if (newTestCases == nil)
    newTestCases = [NSArray array];
  relevantTestCases = [newTestCases retain];
  [self didChangeValueForKey:@"relevantTestCases"];
}

- (void)setDataFileTo:(NSString *)aDataFilePath;
{
  [coverageData autorelease];
  coverageData = [[NSDictionary alloc] initWithContentsOfFile:aDataFilePath];
}

- (void)ignoreLineChangeUntilFileChange;
{
  
}

- (NSArray *)relevantTestCases;
{
  return relevantTestCases;
}

- (NSDictionary *)metaData;
{
  return [coverageData objectForKey:@"metadata"];
}
@end
